@@ -33,6 +33,7 @@ function Epidemic:Epidemic(hospital, contagious_patient)
   self.hospital = hospital
   self.world = self.hospital.world
 
+  -- Epidemic participants
   self.infected_patients = {}
 
   -- The contagious disease the epidemic is based around
@@ -57,16 +58,12 @@ function Epidemic:Epidemic(hospital, contagious_patient)
   self.cover_up_result_fax = {}
 
   -- Set if the user choses the cover up option instead of declaring
-  self.coverup_in_progress = false
+  self.coverup_selected = false
 
   --Cover up timer and amount of intervals the timer has
   self.timer = nil
   self.countdown_intervals = 0
 
-  -- Set when we know if the player has passed/failed the epidemic
-  -- generally used to test if infected patients can still infect others
-  self.result_determined = false
-
   -- Vaccination mode is activated when the icon on the timer
   -- is clicked - used to determine what the cursor should look like
   self.vaccination_mode_active = false
@@ -74,7 +71,7 @@ function Epidemic:Epidemic(hospital, contagious_patient)
   -- For Cheat - Show the contagious icon even before the epidemic is revealed?
   self.cheat_always_show_mood = false
 
-  -- Number of times an infect patient has successfully infected another
+  -- Number of times an infected patients has successfully infected another
   self.total_infections = 0
   -- Number of times any infected patient has tried to infected another - successful or not
   self.attempted_infections = 0
@@ -101,32 +98,24 @@ end
 --[[ The epidemic tick - currently the same rate as the hospital's tick but
 not necessary dependent on it - could potentially be reduced for performance.]]
 function Epidemic:tick()
-  if not self.ready_to_reveal then
-    self:checkIfReadyToReveal()
-  end
-  if not self.result_determined then
-    self:infectOtherPatients()
-  end
-  if self.coverup_in_progress then
-    if not self.result_determined then
-      self:checkNoInfectedPlayerHasLeft()
-      self:markedPatientsCallForVaccination()
-      self:showAppropriateAdviceMessages()
-    end
-    self:tryAnnounceInspector()
-  end
+  self:infectOtherPatients()
+  self:checkIfReadyToReveal()
+  self:showAppropriateAdviceMessages()
+  self:tryAnnounceInspector()
+  self:markedPatientsCallForVaccination()
+  self:checkInfectedLeftHospital()
+  self:checkNoInfectedPatients()
   self:checkPatientsForRemoval()
 end
 
-
 --[[ Adds a new patient to the epidemic who is actively contagious: infected but
   not vaccinated or cured]]
 function Epidemic:addContagiousPatient(patient)
   patient.infected = true
   patient:updateDynamicInfo()
   self.infected_patients[#self.infected_patients + 1] = patient
-  if self.coverup_in_progress or self.cheat_always_show_mood then
-    patient:setMood("epidemy4","activate")
+  if self.coverup_selected or self.cheat_always_show_mood then
+    patient:setInfectedStatus()
   end
 end
 
@@ -137,44 +126,42 @@ function Epidemic:infectOtherPatients()
   --[[ Can an infected patient infect another patient - taking into account
   spread factor as defined in the configuration. Patients must be both in the
   corridors or in the same room - don't infect through walls.
-  @param patient (Patient) already infected patient
-  @param other (Patient) target to possibly infect
+  @param infector (Patient) already infected patient who want to infect
+  @param victim (Patient) target to possibly infect
   @return true if patient can infect other, false otherwise (boolean) ]]
-  local function canInfectOther(patient, other)
-    -- Patient is not infectious.
-    if patient.cured or patient.vaccinated then return false end
+  local function canInfectOther(infector, victim)
+    -- Check is patient-Infector is not infectious.
+    if infector.cured or infector.vaccinated then return false end
 
     -- Don't allow infection outside the hospital grounds
     -- Also check both patients to prevent infecting through outer walls
-    local ppx, ppy = patient.tile_x, patient.tile_y
+    local ppx, ppy = infector.tile_x, infector.tile_y
     if ppx and ppy and not self.hospital:isInHospital(ppx, ppy) then return false end
-    local opx, opy = other.tile_x, other.tile_y
+    local opx, opy = victim.tile_x, victim.tile_y
     if opx and opy and not self.hospital:isInHospital(opx, opy) then return false end
 
-    -- 'other' is already infected or is going home.
-    if other.infected or other.cured or other.vaccinated then return false end
-    if other.is_emergency then return false end -- Don't interact with emergencies.
+    -- 'victim' is already infected or is going home.
+    if victim.infected or victim.cured or victim.vaccinated then return false end
+    -- Don't infect victim if it alredy under another attempt to be infected
+    if victim.under_infection_attempt then return false end
+    -- Don't infect emergencies.
+    if victim.is_emergency then return false end
 
-    -- If the other patient has a different disease OR the other patient's
+    -- If the victim patient has a different disease OR the victim patient's
     -- disease cannot be changed.
-    if patient.disease ~= other.disease and
-        (not other.disease.contagious or other.diagnosed) then return false end
-
-    if other.attempted_to_infect then return false end
+    if infector.disease ~= victim.disease and
+        (not victim.disease.contagious or victim.diagnosed) then return false end
 
     -- Only infect if both are in the same room.
-    return patient:getRoom() == other:getRoom()
+    return infector:getRoom() == victim:getRoom()
   end
 
-  local function infect_other(infected_patient, patient)
-    if infected_patient.disease == patient.disease then
-      self:addContagiousPatient(patient)
-      self.total_infections = self.total_infections + 1
-    else
-      patient:changeDisease(infected_patient.disease)
-      self:addContagiousPatient(patient)
-      self.total_infections = self.total_infections + 1
+  local function infect_other(infector, victim)
+    if infector.disease ~= victim.disease then
+      victim:changeDisease(infector.disease)
     end
+    self:addContagiousPatient(victim)
+    self.total_infections = self.total_infections + 1
   end
 
   -- Scale the chance of spreading the disease infecting spread_factor%
@@ -189,16 +176,16 @@ function Epidemic:infectOtherPatients()
   -- and making any patient they can infect contagious too.
   local entity_map = self.world.entity_map
   if entity_map then
-    for _, infected_patient in ipairs(self.infected_patients) do
+    for _, infector in ipairs(self.infected_patients) do
       local adjacent_patients =
-      entity_map:getPatientsInAdjacentSquares(infected_patient.tile_x, infected_patient.tile_y)
-      for _, patient in ipairs(adjacent_patients) do
-        if canInfectOther(infected_patient, patient) then
-          patient.attempted_to_infect = true
+      entity_map:getPatientsInAdjacentSquares(infector.tile_x, infector.tile_y)
+      for _, potential_victim in ipairs(adjacent_patients) do
+        if canInfectOther(infector, potential_victim) then
+          potential_victim.under_infection_attempt = true
           self.attempted_infections = self.attempted_infections + 1
           if (self.total_infections / self.attempted_infections) <
               (self.spread_factor / spread_scale_factor) then
-            infect_other(infected_patient, patient)
+            infect_other(infector, potential_victim)
           end
         end
       end
@@ -213,6 +200,8 @@ end
  WILL be revealed to the player and may even terminate before they are even
  aware it existed. ]]
 function Epidemic:checkIfReadyToReveal()
+  if self.ready_to_reveal then return end
+
   for _, infected_patient in ipairs(self.infected_patients) do
     if infected_patient.diagnosed then
       self.ready_to_reveal = true
@@ -248,11 +237,11 @@ function Epidemic:announceEndOfEpidemic()
   end
 end
 
---[[ Checks for conditions that could end the epidemic earlier than
- the length of the timer. If an infected patient leaves the
- hospital it's discovered instantly and will result in a fail.]]
-function Epidemic:checkNoInfectedPlayerHasLeft()
-  if self.result_determined then return end
+--[[ Check for conditions that one of the infected and not cured
+patient leaved the hospital. This discover epidemic to public instantly.
+So cover up must end earlier than the length of the timer.]]
+function Epidemic:checkInfectedLeftHospital()
+  if not self:_isCoverUpActive() then return end
 
   -- Check whether a patient has left.
   for _, infected_patient in ipairs(self.infected_patients) do
@@ -261,16 +250,23 @@ function Epidemic:checkNoInfectedPlayerHasLeft()
     if infected_patient.going_home and not infected_patient.cured and
         px and py and not self.hospital:isInHospital(px,py) then
       -- Patient escaped from the hospital, discovery is inevitable.
-      self.result_determined = true
-      if not self.inspector then
-        self:spawnInspector()
-      end
       self:finishCoverUp()
       return
     end
   end
 end
 
+--[[ Check for conditions that no any uncured infected patients
+left in hospital. If so then cover up must end earlier than the
+length of the timer.]]
+function Epidemic:checkNoInfectedPatients()
+  if not self:_isCoverUpActive() then return end
+
+  if self:countInfectedPatients() == 0 then
+    self:finishCoverUp()
+  end
+end
+
 --[[Remove any patients which were already on their way out or died before the
 epidemic was started to be fair on the players so they don't instantly fail.
 Additionally if any patients die during an epidemic we also remove them,
@@ -278,7 +274,7 @@ otherwise a player may never win the epidemic in such a case.]]
 function Epidemic:checkPatientsForRemoval()
   for i = #self.infected_patients, 1, -1 do
     local infected_patient = self.infected_patients[i]
-    if (not self.coverup_in_progress and infected_patient.going_home) or
+    if (not self.coverup_selected and infected_patient.going_home) or
         infected_patient.dead or infected_patient.tile_x == nil then
       table.remove(self.infected_patients,i)
     end
@@ -290,6 +286,17 @@ end
 (@see UIWatch:UIWatch) ]]
 function Epidemic:toggleVaccinationMode()
   self.vaccination_mode_active = not self.vaccination_mode_active
+  self:_updateVaccinationCursor()
+end
+
+--[[ Turn off vaccination cursor mode ]]
+function Epidemic:turnOffVaccinationMode()
+  self.vaccination_mode_active = false
+  self:_updateVaccinationCursor()
+end
+
+--[[ Update how the cursor interacts with the hospital. ]]
+function Epidemic:_updateVaccinationCursor()
   local cursor = self.vaccination_mode_active and "epidemic_hover" or "default"
   self.world.ui:setCursor(self.world.ui.app.gfx:loadMainCursor(cursor))
 end
@@ -301,14 +308,11 @@ end
 function Epidemic:markForVaccination(patient)
   if patient.infected and not patient.vaccinated and
       not patient.marked_for_vaccination then
-    patient.marked_for_vaccination = true
-    patient:setMood("epidemy4","deactivate")
-    patient:setMood("epidemy2","activate")
+    patient:setToReadyForVaccinationStatus()
+    patient.hospital:playSound("vaccin.wav")
   end
 end
 
-
-
 --[[ Counts the number of patients that have been infected that are still
 -- infected
 -- @return infected_count (Integer) the number of patients still infected.]]
@@ -338,7 +342,7 @@ function Epidemic:sendInitialFax()
       {text = _S.fax.epidemic.choices.cover_up, choice = "cover_up_epidemic"},
     },
   }
-  self.world.ui.bottom_panel:queueMessage("epidemy", message, nil, 24*20,2)
+  self.world.ui.bottom_panel:queueMessage("epidemy", message, self, 24*20,2)
 end
 
 --[[ Calculate the fine for having a given number of infected patients
@@ -365,7 +369,6 @@ end
  to cover up it from the initial faxes - ends the epidemic immediately
  after applying fine.]]
 function Epidemic:resolveDeclaration()
-  self.result_determined = true
   self:clearAllInfectedPatients()
 
   --No fax for declaration just apply fines and rep hit
@@ -383,10 +386,7 @@ function Epidemic:clearAllInfectedPatients()
     infected_patient:removeVaccinationCandidateStatus()
     self.world.dispatcher:dropFromQueue(infected_patient)
     infected_patient.vaccinated = true
-    infected_patient:setMood("epidemy1","deactivate")
-    infected_patient:setMood("epidemy2","deactivate")
-    infected_patient:setMood("epidemy3","deactivate")
-    infected_patient:setMood("epidemy4","deactivate")
+    infected_patient:removeAnyEpidemicStatus()
   end
 end
 
@@ -399,29 +399,38 @@ function Epidemic:startCoverUp()
   self.world.ui:addWindow(self.timer)
   -- last chance clean up as entities might have ticked and changed state
   self:checkPatientsForRemoval()
-  self.coverup_in_progress = true
+  self.coverup_selected = true
   --Set the mood icon for all infected patients
   for _, infected_patient in ipairs(self.infected_patients) do
     infected_patient:updateDynamicInfo()
-    infected_patient:setMood("epidemy4","activate")
+    infected_patient:setInfectedStatus()
   end
 end
 
---[[ Ends the cover up of the epidemic with the result to be applied
-later (@see applyOutcome) ]]
+--[[ Ends the cover up epidemic stage ]]
 function Epidemic:finishCoverUp()
-  self.result_determined = true
+  if not self.inspector then
+    self:spawnInspector()
+  end
 
   self.timer:close()
 
-  -- Turn vaccination mode off
-  if self.vaccination_mode_active then
-    self:toggleVaccinationMode()
-  end
+  -- Turn vaccination mode off if enabled
+  self:turnOffVaccinationMode()
+end
 
+--[[ Inspector had arrived at reception desk. Check if
+any infected still in hospital and determine final verdict.]]
+function Epidemic:handleInspectorArrival()
   local still_infected = self:countInfectedPatients()
   self:determineFaxAndFines(still_infected)
   self:clearAllInfectedPatients()
+  self:applyOutcome()
+end
+
+--[[ Epidemic timer time is up ]]
+function Epidemic:coverUpTimeIsUp()
+  self:finishCoverUp()
 end
 
 --[[ Calculates the contents of the fax and the appropriate fines based on the
@@ -488,7 +497,7 @@ function Epidemic:applyOutcome()
     end
     -- Apply fine and reputation hit
     self.hospital:spendMoney(self.coverup_fine,_S.transactions.epidemy_coverup_fine)
-    self.hospital.reputation =  self.hospital.reputation - self.reputation_hit
+    self.hospital.reputation = self.hospital.reputation - self.reputation_hit
   else
     self.hospital:receiveMoney(self.compensation, _S.transactions.compensation)
   end
@@ -539,7 +548,7 @@ end
 --[[ Spawns the inspector who will walk to the reception desk. ]]
 function Epidemic:spawnInspector()
   self.world.ui.adviser:say(_A.information.epidemic_health_inspector)
-  local inspector = self.world:newEntity("Inspector", 2)
+  local inspector = self.world:newEntity("Inspector", 2, 2)
   self.inspector = inspector
   inspector:setType("Inspector")
 
@@ -549,6 +558,18 @@ function Epidemic:spawnInspector()
   inspector:queueAction(SeekReceptionAction())
 end
 
+--[[ Private function to check if a inspector already spawned.
+@return (boolean) true if spawned already, false if not]]
+function Epidemic:_inspectorSpawned()
+  return self.inspector ~= nil
+end
+
+--[[ Private function to check if a cover up is in progress
+@return (boolean) true if cover up in progress, false if not]]
+function Epidemic:_isCoverUpActive()
+  return self.coverup_selected and not self:_inspectorSpawned()
+end
+
 --[[ Is the patient "still" either idle queuing or sitting on a bench
   Typically this is used to determine if a patient can be vaccinated
   @param patient (Patient) the patient we wish to determine if they are static.]]
@@ -672,6 +693,8 @@ end
 --[[ Make the advisor show appropriate messages under certain
   conditions of the epidemic.]]
 function Epidemic:showAppropriateAdviceMessages()
+  if not self:_isCoverUpActive() then return end
+
   if self.countdown_intervals then
     if not self.has_said_hurry_up and self:countInfectedPatients() > 0 and
         -- If only 1/4 of the countdown_intervals remaining on the timer
@@ -696,16 +719,42 @@ function Epidemic:hasNoInfectedPatients()
 end
 
 function Epidemic:tryAnnounceInspector()
+  if not self.coverup_selected then return end
+  if not self:_inspectorSpawned() then return end
+
   local inspector = self.inspector
-  if inspector and not inspector.has_been_announced and
-      self.hospital:isInHospital(inspector.tile_x, inspector.tile_y) then
+  if not inspector.has_been_announced and
+    self.hospital:isInHospital(inspector.tile_x, inspector.tile_y) then
     inspector:announce()
     inspector.has_been_announced = true
   end
 end
 
+--[[ For Cheat - Cancel the epidemic. ]]
+function Epidemic:cancelEpidemic()
+  -- Remove init epidemic fax
+  self.world.ui.bottom_panel:removeMessage(self)
+  -- Turn vaccination mode off if enabled
+  self:turnOffVaccinationMode()
+  -- Remove epidemic timer
+  if self.timer ~= nil then
+    self.timer:close()
+  end
+  -- Send inspector home if spawned
+  if self.inspector ~= nil then
+    self.inspector:goHome()
+  end
+  -- Clear epidemic patients
+  self:clearAllInfectedPatients()
+  self.infected_patients = {}
+end
+
 function Epidemic:afterLoad(old, new)
   if old < 106 then
     self.level_config = nil
   end
+  if old < 212 then
+    self.coverup_selected = self.coverup_in_progress
+    self.coverup_in_progress = nil
+  end
 end
